package rition.service.collector;

import com.github.yitter.idgen.YitIdHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;
import rition.common.data.dto.MetricDataDto;
import rition.common.data.enums.CommonEntityStatus;
import rition.service.collector.configure.KafkaConfigure;
import rition.service.collector.dao.entity.MetricRecordEntity;
import rition.service.collector.dao.mapper.MetricRecordMapper;

import java.time.Instant;
import java.util.ArrayList;

/**
 * DataCollectorService 指标数据收集类，负责收集来自云主机探针上报的数据，并将其入库等
 */
@Slf4j
@Service
public class MetricCollectingService {

    private final KafkaTemplate<String, MetricDataDto> kafkaTemplate;
    private final KafkaConfigure kafkaConfigure;

    private final String collectedDataTopic;
    private final MetricRecordMapper metricRecordMapper;

    public MetricCollectingService(KafkaTemplate<String, MetricDataDto> kafkaTemplate,
                                   KafkaConfigure kafkaConfigure, MetricRecordMapper metricRecordMapper) {
        this.kafkaTemplate = kafkaTemplate;
        this.kafkaConfigure = kafkaConfigure;
        this.collectedDataTopic = kafkaConfigure.getDataCollecting().getTopic();
        this.metricRecordMapper = metricRecordMapper;
    }

    private static final int MAX_BATCH_INSERT_SIZE = 512;

    /**
     * 接收处理好的监控指标数据
     *
     * @param collectedMetricData 监控指标数据，protobuf对象
     */
    public void receiveData(MetricDataDto collectedMetricData) {
        // 对大于MAX_BATCH_INSERT_SIZE大小的数据进行分组的批量插入
        var metricDataList = collectedMetricData.getDataList();
        if (metricDataList.size() <= MAX_BATCH_INSERT_SIZE) {
            var metricRecordEntityList = new ArrayList<MetricRecordEntity>(metricDataList.size());
            for (MetricDataDto.MetricData metricData : metricDataList) {
                MetricRecordEntity entity = this.convert(metricData);

                metricRecordEntityList.add(entity);
            }

            kafkaTemplate.send(this.collectedDataTopic, collectedMetricData);
            metricRecordMapper.insertBatchSomeColumn(metricRecordEntityList);
        } else {
            var batch = metricDataList.size() / MAX_BATCH_INSERT_SIZE;
            if (metricDataList.size() % MAX_BATCH_INSERT_SIZE != 0) {
                batch++;
            }

            var metricRecordEntityList = new ArrayList<MetricRecordEntity>(MAX_BATCH_INSERT_SIZE);
            for (int i = 0; i < batch; i++) {
                metricRecordEntityList.clear();
                var from = i * MAX_BATCH_INSERT_SIZE;
                var to = (i + 1) * MAX_BATCH_INSERT_SIZE;
                var subList = metricDataList.subList(from, Math.min(metricDataList.size(), to));
                for (MetricDataDto.MetricData metricData : subList) {
                    MetricRecordEntity entity = this.convert(metricData);
                    metricRecordEntityList.add(entity);
                }

                kafkaTemplate.send(this.collectedDataTopic, MetricDataDto.builder().dataList(subList).build());
                metricRecordMapper.insertBatchSomeColumn(metricRecordEntityList);
            }
        }
    }

    public MetricRecordEntity convert(MetricDataDto.MetricData metricData) {
        MetricRecordEntity entity = new MetricRecordEntity();
        entity.setId(YitIdHelper.nextId());
        entity.setInstanceId(metricData.getTags().get("instanceId"));
        entity.setItem(metricData.getMetric());
        entity.setValue(metricData.getValue());
        entity.setStatus(CommonEntityStatus.STATUS_NORMAL.getValue());

        var time = Instant.ofEpochMilli(metricData.getTimestamp());
        entity.setTime(time);
        entity.setUpdateTime(time);

        return entity;
    }
}
